@@ -1,6 +1,6 @@
 '''
 [LICENSE]
-Copyright (c) 2018 Alliance for Sustainable Energy, LLC, All Rights Reserved
+Copyright (c) 2019 Alliance for Sustainable Energy, LLC, All Rights Reserved
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
@@ -18,6 +18,7 @@ THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER(S) AND ANY CONTRIBUTORS "AS IS
 
 [/LICENSE]
 '''
+
 from __future__ import print_function, division, absolute_import
 
 import argparse
@@ -30,7 +31,8 @@ from uuid import uuid4
 
 from jinja2 import Environment, FileSystemLoader
 from layerstack.args import ArgList, KwargDict, ArgMode
-from layerstack import checksum, LayerStackError, start_console_log
+from layerstack import (DEFAULT_LOG_FORMAT, LayerStackError, checksum, 
+    start_console_log)
 
 
 logger = logging.getLogger(__name__)
@@ -64,11 +66,6 @@ class LayerBase(object):
         Each layer must define its positional arguments by populating and 
         returning an ArgList object.
 
-        Parameters
-        ----------
-        **kwargs
-            application-internal kwargs
-
         Returns
         -------
         ArgList
@@ -84,11 +81,6 @@ class LayerBase(object):
         Each layer must define its keyword arguments by populating and returning
         a KwargDict object.
 
-        Parameters
-        ----------
-        **kwargs
-            application-internal kwargs
-
         Returns
         -------
         KwargDict
@@ -122,8 +114,7 @@ class LayerBase(object):
     # TODO: Split main into parser and execution. Should be able to re-use
     # parser-part for workflows.
     @classmethod
-    def main(cls,
-             log_format='%(asctime)s|%(levelname)s|%(name)s|\n\t%(message)s'):
+    def main(cls, log_format=DEFAULT_LOG_FORMAT):
         """
         Single-layer command-line interface entry point.
 
@@ -159,7 +150,17 @@ class LayerBase(object):
         if not os.path.isdir(cli_args.run_dir):
             raise LayerStackError("The run directory '{}' does not exist."
                                   .format(os.path.abspath(cli_args.run_dir)))
-        cls._main_apply(cli_args, arg_list, kwarg_dict)
+        old_cur_dir = os.getcwd()
+        os.chdir(cli_args.run_dir)
+
+        try:
+            cls._main_apply(cli_args, arg_list, kwarg_dict)
+        # switch back to initial directory
+            os.chdir(old_cur_dir)
+        except:
+            os.chdir(old_cur_dir)
+            raise
+
         sys.exit()
 
     @classmethod
@@ -172,8 +173,7 @@ class LayerBase(object):
         str
             Layer description or 'Apply Layer {}'.format(cls.name) by default
         """
-        return cls.desc if cls.desc is not None else "Apply Layer \
-'{}'".format(cls.name)
+        return cls.desc if cls.desc is not None else "Apply Layer '{}'".format(cls.name)
 
     @classmethod
     def _main_apply(cls, cli_args, arg_list, kwarg_dict):
@@ -196,7 +196,8 @@ class LayerBase(object):
         assert arg_list.mode == ArgMode.USE
         assert kwarg_dict.mode == ArgMode.USE
         from layerstack.stack import Stack
-        return cls.apply(Stack(), *arg_list, **kwarg_dict)
+        # TODO: Fix KwargDict so **kwarg_dict works natively
+        return cls.apply(Stack(), *arg_list, **{k: v for k, v in kwarg_dict.items()})
 
     @classmethod
     def _add_positional_arguments(cls, parser): 
@@ -209,6 +210,10 @@ class LayerBase(object):
 class ModelLayerBase(LayerBase):
     """
     Abstract base class for user-defined layers that operate on a model.
+
+    .. automethod:: _check_model_type
+    .. automethod:: _load_model
+    .. automethod:: _save_model
     """
 
     @classmethod
@@ -220,8 +225,6 @@ class ModelLayerBase(LayerBase):
         ----------
         model : None or a model
             model to be operated on
-        **kwargs
-            Internal kwargs
 
         Returns
         -------
@@ -239,8 +242,6 @@ class ModelLayerBase(LayerBase):
         ----------
         model
             model to be operated on
-        **kwargs
-            Internal kwargs
 
         Returns
         -------
@@ -252,12 +253,18 @@ class ModelLayerBase(LayerBase):
     @classmethod
     def _check_model_type(cls, model):
         """
-        Check model type to ensure it is operable
+        Check model type to ensure it is operable. Raises an exception if the
+        model is the wrong type or some other error is encountered. Derived 
+        classes should implement this method.
 
         Parameters
         ----------
         model
             model to be operated on
+
+        Returns
+        -------
+        None
         """
         # Check to make sure model is of the proper type
         pass
@@ -265,18 +272,22 @@ class ModelLayerBase(LayerBase):
     @classmethod
     def apply(cls, stack, model, *args, **kwargs):
         """
-        Run layer
+        Run this layer in the context of the stack, with positional and keyword
+        arguments. In general in user-defined layers (classes derived from 
+        LayerBase), \*args and \*\*kwargs should be replaced by the actual 
+        positional argument names (defined in the args method) and keyword 
+        argument name, default value pairs (defined in the kwargs method).
 
         Parameters
         ----------
         stack : 'Stack'
             Stack class instance in which the layer is being run
         model
-            model to be operated on
+            model this layer will operate on
         *args
-            The layer's args
+            The layer's positional arguments
         **kwargs
-            The layer's kwargs
+            The layer's keyword arguments
 
         Returns
         -------
@@ -332,33 +343,42 @@ model".format(cls.name)
             updated model
         """
         model = cls._load_model(cli_args.model)
-        from .stack import Stack
-        return cls.apply(Stack(), model, *arg_list, **kwarg_dict)
+        assert arg_list.mode == ArgMode.USE
+        assert kwarg_dict.mode == ArgMode.USE
+        from layerstack.stack import Stack
+        # TODO: Fix KwargDict so **kwarg_dict works natively
+        return cls.apply(Stack(), model, *arg_list, **{k: v for k, v in kwarg_dict.items()})
 
     @classmethod
     def _load_model(cls, model_path):
         """
-        Load model from cli
+        Load model from cli. Derived classes must implement this method if they
+        support loading from disk.
 
         Parameters
         ----------
         model_path : 'str'
             Path from which to load model
+
+        Returns
+        -------
+        model
         """
-        # Method to load model
         pass
 
     @classmethod
-    def _save_model(cls, model_path):
+    def _save_model(cls, model, model_path):
         """
-        Save model from cli
+        Save model from cli. Derived classes must implement this method if they 
+        support saving the model out to disk.
 
         Parameters
         ----------
+        model 
+            model to be saved to disk
         model_path : 'str'
             Path to save model to
         """
-        # Method to save model
         pass
 
 
@@ -465,11 +485,13 @@ already exists."
             kwargs to pass to layer template
         """
         def class_name(name):
-            result = name.title()
-            replacements = [(" ", ""),
-                            ("-", "")]
-            for old, new in replacements:
-                result = result.replace(old, new)
+            words = [y for x in name.split() for y in x.split('-')]
+            result = ''
+            for word in words:
+                if word.isupper():
+                    result += word
+                else:
+                    result += word.title()
             return result
 
         kwargs = {}
@@ -483,6 +505,22 @@ already exists."
         if desc is not None:
             kwargs['desc'] = desc
 
+        def class_doc_str(aclass, doc_str=''):
+            if issubclass(aclass, LayerBase):
+                result = '\n    {}\n    {}'.format(aclass.__name__, '=' * len(aclass.__name__))
+                result += aclass.__doc__
+
+                for base_class in aclass.__bases__:
+                    result = class_doc_str(base_class, doc_str=(result + doc_str))
+
+                return result
+            return doc_str
+
+        kwargs['layer_base_class_doc'] = class_doc_str(layer_base_class)
+        kwargs['layer_base_class_args_doc'] = layer_base_class.args.__doc__
+        kwargs['layer_base_class_kwargs_doc'] = layer_base_class.kwargs.__doc__
+        kwargs['layer_base_class_apply_doc'] = layer_base_class.apply.__doc__
+
         main_opts = ""
         lead = None
         for ln in LayerBase.main.__doc__.split("\n"):
